package com.fr.plugin.expand.sheet;

import com.fr.base.FRContext;
import com.fr.base.TableData;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.stable.script.CalculatorProvider;
import com.tptj.plugin.hg.bean.ExpandRequest;
import com.tptj.plugin.hg.bean.ExpandSheet;
import com.tptj.plugin.hg.impl.AbstractExpandProvider;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Demo
 *
 * @author maxinchun
 * @date 2020/1/20 10:25
 */
@FunctionRecorder
public class Demo extends AbstractExpandProvider {


    @Override
    public boolean accept(Map<String, Object> map, String bookpath) {
        //demo方案只对demo.cpt生效，若多个方案都满足，则只执行最先匹配的方案，所以原则上不要多个方案同时生效
        return "new_demo.cpt".equals(bookpath) || "/new_demo.cpt".equals(bookpath);
    }

    @Override
    public List<ExpandRequest> getExpand(
            Map<String, Object> parameters, Map<String, TableData> resources,
            List<String> sheetNames, CalculatorProvider cal) {
        List<ExpandRequest> result = new ArrayList<ExpandRequest>();
        try {
            //前端传递过来的参数是被编码过的，所以这里需要解码比较一下，并生成销售员的数组
            String[] sellers = getSellers(parameters);
            //ExpandRequest就是定义需要对cpt的哪个sheet进行扩展
            //此例中我们对demo.cpt中名为sheet1的sheet进行扩展
            ExpandRequest request = new ExpandRequest("sheet1");
            for (String seller : sellers) {
                //每一个销售员定义扩展一个sheet
                //使用销售员的名字作为sheet名
                ExpandSheet sheet = new ExpandSheet(seller);
                //定义这个扩展sheet的数据集需要用什么参数值重新计算
                //将要数据集添加到当前的扩展记录sheet中
                Map<String, Object> nparams = new HashMap<String, Object>();
                nparams.put("销售员", seller);
                sheet.addDatasource("销量", nparams);
                //对原sheet记录一次扩展：表示这个sheet是用sheet1扩展的
                request.addExpandSheet(sheet);
            }
            //如果要对多个原有的sheet进行扩展~就添加多个
            result.add(request);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("sheet 扩展异常", e);
        }
        return result;
    }


    /**
     * 从参数表中，获取需要扩展的销售员的数组
     *
     * @param parameters
     * @return
     */
    private String[] getSellers(Map<String, Object> parameters) {
        try {
            Object val = parameters.get(URLEncoder.encode("销售员", "utf-8"));
            if (!(val instanceof String)) {
                return new String[0];
            }
            String vstr = val.toString();
            String[] sellers = new String[]{vstr};
            if (vstr.indexOf(",") != -1) {
                sellers = vstr.split(",");
            }
            return sellers;
        } catch (Exception e) {
            FRContext.getLogger().error("获取销售员出错！", e);
        }
        return new String[0];
    }
}
